@@ -30,41 +30,6 @@
 #include "afterconf.h"
 
 
-TermDef       WindowBoxTerms[] = {
-    {TF_NO_MYNAME_PREPENDING, "WindowBox", 9, TT_QUOTED_TEXT, WINDOWBOX_START_ID, NULL},
-    {TF_NO_MYNAME_PREPENDING, "Area", 4,              TT_GEOMETRY,WINDOWBOX_Area_ID   	  	 , NULL},
-    {TF_NO_MYNAME_PREPENDING, "Virtual", 7,           TT_FLAG,    WINDOWBOX_Virtual_ID       , NULL},
-    {TF_NO_MYNAME_PREPENDING, "MinWidth", 8,          TT_INTEGER, WINDOWBOX_MinWidth_ID	  	 , NULL},
-    {TF_NO_MYNAME_PREPENDING, "MinHeight", 9,         TT_INTEGER, WINDOWBOX_MinHeight_ID	 , NULL},
-    {TF_NO_MYNAME_PREPENDING, "MaxWidth", 8,          TT_INTEGER, WINDOWBOX_MaxWidth_ID	  	 , NULL},
-    {TF_NO_MYNAME_PREPENDING, "MaxHeight", 9,         TT_INTEGER, WINDOWBOX_MaxHeight_ID	 , NULL},
-    {TF_NO_MYNAME_PREPENDING, "FirstTry", 8,          TT_INTEGER, WINDOWBOX_FirstTry_ID	  	 , NULL},
-    {TF_NO_MYNAME_PREPENDING, "ThenTry", 7,           TT_INTEGER, WINDOWBOX_ThenTry_ID 	  	 , NULL},
-    {TF_NO_MYNAME_PREPENDING, "Desk", 4,              TT_INTEGER, WINDOWBOX_Desk_ID          , NULL},
-    {TF_NO_MYNAME_PREPENDING, "MinLayer", 8,          TT_INTEGER, WINDOWBOX_MinLayer_ID      , NULL},
-    {TF_NO_MYNAME_PREPENDING, "MaxLayer", 8,          TT_INTEGER, WINDOWBOX_MaxLayer_ID      , NULL},
-    {TF_NO_MYNAME_PREPENDING, "VerticalPriority", 16, TT_FLAG,    WINDOWBOX_VerticalPriority_ID, NULL},
-	{TF_NO_MYNAME_PREPENDING, "ReverseOrder", 12,     TT_FLAG,    WINDOWBOX_ReverseOrder_ID    , NULL},
-    {TF_NO_MYNAME_PREPENDING |
-	 TF_SYNTAX_TERMINATOR,    "~WindowBox", 10, TT_FLAG, WINDOWBOX_DONE_ID, NULL},
-	{0, NULL, 0, 0, 0}
-};
-
-SyntaxDef     WindowBoxSyntax = {
-	'\n',
-	'\0',
-	WindowBoxTerms,
-    7,                                         /* hash size */
-	'\t',
-	"",
-	"\t",
-	"Feel WindowBox definition",
-	"FeelWindowBox",
-	"defines placement policy for certain type of windows",
-	NULL,
-	0
-};
-
 flag_options_xref WindowBoxFlagsXref[] = {
     { ASA_Virtual			, WINDOWBOX_Virtual_ID, 0},
     { ASA_VerticalPriority	, WINDOWBOX_VerticalPriority_ID, 0},
@@ -82,100 +47,6 @@ flag_options_xref WindowBoxFlagsXref[] = {
 extern SyntaxDef     PopupFuncSyntax ;
 void input_context_destroy(ASHashableValue value, void *data);
 
-TermDef FeelTerms[] =
-{
-  /* focus : */
-  {TF_NO_MYNAME_PREPENDING, "ClickToFocus",12           , TT_FLAG       , FEEL_ClickToFocus_ID          , NULL},
-  {TF_NO_MYNAME_PREPENDING, "SloppyFocus",11            , TT_FLAG       , FEEL_SloppyFocus_ID           , NULL},
-  {TF_NO_MYNAME_PREPENDING, "AutoFocus",9               , TT_FLAG       , FEEL_AutoFocus_ID             , NULL},
-  /* decorations : */
-  {TF_NO_MYNAME_PREPENDING, "DecorateTransients",18     , TT_FLAG       , FEEL_DecorateTransients_ID    , NULL},
-  /* placement : */
-  {TF_NO_MYNAME_PREPENDING, "DontMoveOff",11            , TT_FLAG       , FEEL_DontMoveOff_ID           , NULL},
-  {TF_NO_MYNAME_PREPENDING|TF_DEPRECIATED, "NoPPosition",11            , TT_FLAG       , FEEL_NoPPosition_ID           , NULL},
-  {TF_NO_MYNAME_PREPENDING, "SmartPlacement",14         , TT_FLAG       , FEEL_SmartPlacement_ID        , NULL},
-  {TF_NO_MYNAME_PREPENDING, "RandomPlacement",15        , TT_FLAG       , FEEL_RandomPlacement_ID       , NULL},
-  {TF_NO_MYNAME_PREPENDING, "StubbornPlacement",17      , TT_FLAG       , FEEL_StubbornPlacement_ID     , NULL},
-  {TF_NO_MYNAME_PREPENDING, "MenusHigh",9               , TT_FLAG       , FEEL_MenusHigh_ID             , NULL},
-  {TF_NO_MYNAME_PREPENDING, "CenterOnCirculate",17      , TT_FLAG       , FEEL_CenterOnCirculate_ID     , NULL},
-  /* icons : */
-  {TF_NO_MYNAME_PREPENDING, "SuppressIcons",13          , TT_FLAG       , FEEL_SuppressIcons_ID         , NULL},
-  {TF_NO_MYNAME_PREPENDING, "IconTitle",9               , TT_FLAG       , FEEL_IconTitle_ID             , NULL},
-  {TF_NO_MYNAME_PREPENDING, "KeepIconWindows",15        , TT_FLAG       , FEEL_KeepIconWindows_ID       , NULL},
-  {TF_NO_MYNAME_PREPENDING, "StickyIcons",11            , TT_FLAG       , FEEL_StickyIcons_ID           , NULL},
-  {TF_NO_MYNAME_PREPENDING, "StubbornIcons",13          , TT_FLAG       , FEEL_StubbornIcons_ID         , NULL},
-  {TF_NO_MYNAME_PREPENDING, "StubbornIconPlacement",21  , TT_FLAG       , FEEL_StubbornIconPlacement_ID , NULL},
-  {TF_NO_MYNAME_PREPENDING, "CirculateSkipIcons",18     , TT_FLAG       , FEEL_CirculateSkipIcons_ID    , NULL},
-  /* backing store/save unders : */
-  {TF_NO_MYNAME_PREPENDING, "BackingStore",12           , TT_FLAG       , FEEL_BackingStore_ID          , NULL},
-  {TF_NO_MYNAME_PREPENDING, "AppsBackingStore",16       , TT_FLAG       , FEEL_AppsBackingStore_ID      , NULL},
-  {TF_NO_MYNAME_PREPENDING, "SaveUnders",10             , TT_FLAG       , FEEL_SaveUnders_ID            , NULL},
-  /* pageing */
-  {TF_NO_MYNAME_PREPENDING, "PagingDefault",13          , TT_FLAG       , FEEL_PagingDefault_ID         , NULL},
-  {TF_NO_MYNAME_PREPENDING, "AutoTabThroughDesks",19    , TT_FLAG       , FEEL_AutoTabThroughDesks_ID   , NULL},
-
-  {TF_NO_MYNAME_PREPENDING, "ClickTime",9               , TT_UINTEGER   , FEEL_ClickTime_ID         , NULL},
-  {TF_NO_MYNAME_PREPENDING, "OpaqueMove",10             , TT_UINTEGER   , FEEL_OpaqueMove_ID        , NULL},
-  {TF_NO_MYNAME_PREPENDING, "OpaqueResize",12           , TT_UINTEGER   , FEEL_OpaqueResize_ID      , NULL},
-  {TF_NO_MYNAME_PREPENDING, "AutoRaise",9               , TT_UINTEGER   , FEEL_AutoRaise_ID         , NULL},
-  {TF_NO_MYNAME_PREPENDING, "AutoReverse",11            , TT_UINTEGER   , FEEL_AutoReverse_ID       , NULL},
-  {TF_NO_MYNAME_PREPENDING, "DeskAnimationType",17      , TT_UINTEGER   , FEEL_DeskAnimationType_ID , NULL},
-  {TF_NO_MYNAME_PREPENDING, "ShadeAnimationSteps",19    , TT_UINTEGER   , FEEL_ShadeAnimationSteps_ID,NULL},
-
-  {TF_NO_MYNAME_PREPENDING, "DeskAnimationSteps",18     , TT_UINTEGER   , FEEL_DeskAnimationSteps_ID, NULL},
-
-  {TF_NO_MYNAME_PREPENDING, "XorValue",8                , TT_INTEGER    , FEEL_XorValue_ID          , NULL},
-  {TF_NO_MYNAME_PREPENDING, "Xzap",4                    , TT_INTEGER    , FEEL_Xzap_ID              , NULL},
-  {TF_NO_MYNAME_PREPENDING, "Yzap",4                    , TT_INTEGER    , FEEL_Yzap_ID              , NULL},
-  {TF_INDEXED|TF_NO_MYNAME_PREPENDING,
-                            "Cursor",6                  , TT_INTEGER    , FEEL_Cursor_ID            , NULL},
-  {TF_INDEXED|TF_NO_MYNAME_PREPENDING,
-                            "CustomCursor",12           , TT_CURSOR     , FEEL_CustomCursor_ID      , NULL},
-
-  {TF_NO_MYNAME_PREPENDING, "ClickToRaise",12           , TT_BITLIST    , FEEL_ClickToRaise_ID      , NULL},
-
-  {TF_NO_MYNAME_PREPENDING, "EdgeScroll",10             , TT_INTARRAY   , FEEL_EdgeScroll_ID        , NULL},
-  {TF_NO_MYNAME_PREPENDING, "EdgeResistance",14         , TT_INTARRAY   , FEEL_EdgeResistance_ID    , NULL},
-
-  /* this stuff requires subsyntaxes : */
-  {TF_NO_MYNAME_PREPENDING, "Popup",5                   , TT_QUOTED_TEXT, FEEL_Popup_ID             , &PopupFuncSyntax},
-  {TF_NO_MYNAME_PREPENDING, "Function",8                , TT_QUOTED_TEXT, FEEL_Function_ID          , &PopupFuncSyntax},
-
-  /* mouse and key bindings require special processing : */
-  {TF_SPECIAL_PROCESSING|TF_NO_MYNAME_PREPENDING,
-                            "Mouse",5                   , TT_BINDING    , FEEL_Mouse_ID             , &FuncSyntax},
-  {TF_SPECIAL_PROCESSING|TF_NO_MYNAME_PREPENDING,
-                            "Key",3                     , TT_BINDING    , FEEL_Key_ID               , &FuncSyntax},
-
-  {TF_NO_MYNAME_PREPENDING, "ShadeAnimationSteps", 19, TT_UINTEGER, 	FEEL_ShadeAnimationSteps_ID, NULL},
-  {TF_NO_MYNAME_PREPENDING, "CoverAnimationSteps", 19, TT_UINTEGER, 	FEEL_CoverAnimationSteps_ID, NULL },
-  {TF_NO_MYNAME_PREPENDING, "CoverAnimationType", 18,  TT_UINTEGER, 	FEEL_CoverAnimationType_ID, NULL },
-
-
-    /* obsolete stuff : */
-  {TF_OBSOLETE|TF_NO_MYNAME_PREPENDING, "MWMFunctionHints",16       , TT_FLAG       , FEEL_MWMFunctionHints_ID  , NULL},
-  {TF_OBSOLETE|TF_NO_MYNAME_PREPENDING, "MWMDecorHints",13          , TT_FLAG       , FEEL_MWMDecorHints_ID     , NULL},
-  {TF_OBSOLETE|TF_NO_MYNAME_PREPENDING, "MWMHintOverride",15        , TT_FLAG       , FEEL_MWMHintOverride_ID   , NULL},
-
-  {0, NULL, 0, 0, 0}
-};
-
-SyntaxDef FeelSyntax =
-{
-  '\n',
-  '\0',
-  FeelTerms,
-  0,				/* use default hash size */
-  '\t',
-  "",
-  "\t",
-  "Feel",
-  "Feel",
-  "AfterStep feel configuration",
-  NULL,
-  0
-};
-
 flag_options_xref FeelFlagsXref[] = {
      /* focus : */
     {ClickToFocus          , FEEL_ClickToFocus_ID          , 0},
@@ -189,7 +60,7 @@ flag_options_xref FeelFlagsXref[] = {
     {SMART_PLACEMENT        , FEEL_SmartPlacement_ID        , 0},
     {RandomPlacement       , FEEL_RandomPlacement_ID       , 0},
     {StubbornPlacement     , FEEL_StubbornPlacement_ID     , 0},
-    {MenusHigh             , FEEL_MenusHigh_ID             , 0},
+//    {MenusHigh             , FEEL_MenusHigh_ID             , 0},
     {CenterOnCirculate     , FEEL_CenterOnCirculate_ID     , 0},
      /* icons : */
     {SuppressIcons         , FEEL_SuppressIcons_ID         , 0},
@@ -210,51 +81,6 @@ flag_options_xref FeelFlagsXref[] = {
 	{0, 0, 0}
 };
 
-TermDef AutoExecTerms[] =
-{
-  {TF_NO_MYNAME_PREPENDING, "Function",8                , TT_QUOTED_TEXT, FEEL_Function_ID          , &PopupFuncSyntax},
-  {0, NULL, 0, 0, 0}
-};
-
-SyntaxDef AutoExecSyntax =
-{
-  '\n',
-  '\0',
-  AutoExecTerms,
-  0,				/* use default hash size */
-  ' ',
-  "",
-  "\t",
-  "AfterStep Autoexec (startup/restart sequences)",
-  "AutoExec",
-  "functions to be executed by AfterStep on startup/shutdown",
-  NULL,
-  0
-};
-
-TermDef ThemeTerms[] =
-{
-  {TF_NO_MYNAME_PREPENDING, "Function",8                , TT_QUOTED_TEXT, FEEL_Function_ID          , &PopupFuncSyntax},
-  {0, NULL, 0, 0, 0}
-};
-
-SyntaxDef ThemeSyntax =
-{
-  '\n',
-  '\0',
-  ThemeTerms,
-  0,				/* use default hash size */
-  ' ',
-  "",
-  "\t",
-  "theme installation script",
-  "Theme",
-  "AfterStep theme file",
-  NULL,
-  0
-};
-
-
 
 /**************************************************************************************
  * WindowBox code :
@@ -328,12 +154,18 @@ ProcessWindowBoxOptions (FreeStorageElem * options)
                     set_flags( aswbox->set_flags, ASA_MaxHeightSet);
                     break ;
                 case WINDOWBOX_FirstTry_ID        :
-                    aswbox->main_strategy = item.data.integer ;
-                    set_flags( aswbox->set_flags, ASA_MainStrategySet);
+					if( options->sub != NULL && options->sub->term->id >= FEEL_PLACEMENT_START_ID ) 
+					{	
+                    	aswbox->main_strategy = options->sub->term->id - FEEL_PLACEMENT_START_ID ;
+                    	set_flags( aswbox->set_flags, ASA_MainStrategySet);
+					}
                     break ;
                 case WINDOWBOX_ThenTry_ID         :
-                    aswbox->backup_strategy = item.data.integer ;
-                    set_flags( aswbox->set_flags, ASA_BackupStrategySet);
+					if( options->sub != NULL && options->sub->term->id >= FEEL_PLACEMENT_START_ID ) 
+					{	
+                    	aswbox->backup_strategy = options->sub->term->id - FEEL_PLACEMENT_START_ID ;
+                    	set_flags( aswbox->set_flags, ASA_BackupStrategySet);
+					}
                     break ;
                 case WINDOWBOX_Desk_ID            :
                     aswbox->desk = item.data.integer ;
