@@ -24,7 +24,7 @@
 #undef DEBUG_COMPRESS
 #undef DEBUG_THRESHOLD
 #endif
-#undef DO_CLOCKING
+#define DO_CLOCKING
 
 #ifdef _WIN32
 #include "win32/config.h"
@@ -520,7 +520,10 @@ compress_stored_data( ASStorage *storage, CARD8 *data, int size, ASFlagType *fla
 	CARD8  *buffer = data ;
 	size_t 	buf_size = size ; 
 	
-	if( get_flags( *flags, ASStorage_RLEDiffCompress ) && size > ASStorageSlot_SIZE)
+	if( size < ASStorageSlot_SIZE ) 
+		clear_flags( *flags, ASStorage_RLEDiffCompress );
+
+	if( get_flags( *flags, ASStorage_RLEDiffCompress ) )
 	{
 		int uncompressed_size = size ;
 
@@ -1380,20 +1383,21 @@ card8_threshold( ASStorageDstBuffer *dst, void *src, size_t size)
 
 static inline int  
 fetch_data_int( ASStorage *storage, ASStorageID id, ASStorageDstBuffer *buffer, int offset, int buf_size, CARD8 bitmap_value, 
-		  		data_cpy_func_type cpy_func)
+		  		data_cpy_func_type cpy_func, int *original_size)
 {
 	ASStorageSlot *slot = find_storage_slot( find_storage_block( storage, id ), id );
 	LOCAL_DEBUG_OUT( "slot = %p", slot );
 	if( slot )
 	{
 		int uncomp_size = slot->uncompressed_size ;
+		*original_size = uncomp_size ;
 		if( get_flags( slot->flags, ASStorage_Reference) )
 		{
 			ASStorageID target_id = 0;
 			memcpy( &target_id, &(slot->data[0]), sizeof( ASStorageID ));				   
 			LOCAL_DEBUG_OUT( "target_id = %lX", target_id );
 			if( target_id != 0 ) 
-				return fetch_data_int(storage, target_id, buffer, offset, buf_size, bitmap_value, cpy_func);
+				return fetch_data_int(storage, target_id, buffer, offset, buf_size, bitmap_value, cpy_func, original_size);
 			else
 				return 0;
 		}	 
@@ -1409,6 +1413,10 @@ fetch_data_int( ASStorage *storage, ASStorageID id, ASStorageDstBuffer *buffer,
 													uncomp_size, slot->flags, bitmap_value );
 			while( offset > uncomp_size ) offset -= uncomp_size ; 
 			while( offset < 0 ) offset += uncomp_size ; 
+			
+			if( get_flags( slot->flags, ASStorage_NotTileable ) )
+				if( buf_size > uncomp_size - offset ) 
+					buf_size = uncomp_size - offset ;
 			if( offset > 0 ) 
 			{
 				int to_copy = uncomp_size-offset ; 
@@ -1417,6 +1425,7 @@ fetch_data_int( ASStorage *storage, ASStorageID id, ASStorageDstBuffer *buffer,
 				cpy_func( buffer, tmp+offset, to_copy ); 															
 				buffer->offset = to_copy ;
 			}
+			LOCAL_DEBUG_OUT( "offset = %d", buffer->offset );
 			while( buffer->offset < buf_size ) 
 			{
 				int to_copy = buf_size - buffer->offset ; 
@@ -1556,34 +1565,42 @@ store_data_tinted(ASStorage *storage, CARD8 *data, int size, ASFlagType flags, C
 
 
 int  
-fetch_data(ASStorage *storage, ASStorageID id, CARD8 *buffer, int offset, int buf_size, CARD8 bitmap_value)
+fetch_data(ASStorage *storage, ASStorageID id, CARD8 *buffer, int offset, int buf_size, CARD8 bitmap_value, int *original_size)
 {
+	int dumm ; 
 	if( storage == NULL ) 
 		storage = get_default_asstorage();
-	
+
+	if( original_size == NULL ) 
+		original_size = &dumm ;
+	*original_size = 0;
 	if( storage != NULL && id != 0 )
 	{	
 		ASStorageDstBuffer buf ; 
 		buf.offset = 0 ; 
 		buf.buffer = buffer ;
-		return fetch_data_int( storage, id, &buf, offset, buf_size, bitmap_value, card8_card8_cpy );
+		return fetch_data_int( storage, id, &buf, offset, buf_size, bitmap_value, card8_card8_cpy, original_size );
 	}
 	return 0 ;	 
 }
 
 int  
-fetch_data32(ASStorage *storage, ASStorageID id, CARD32 *buffer, int offset, int buf_size, CARD8 bitmap_value)
+fetch_data32(ASStorage *storage, ASStorageID id, CARD32 *buffer, int offset, int buf_size, CARD8 bitmap_value, int *original_size)
 {
+	int dumm ;
 	if( storage == NULL ) 
 		storage = get_default_asstorage();
 	
+	if( original_size == NULL ) 
+		original_size = &dumm ;
+	*original_size = 0;
 	if( storage != NULL && id != 0 )
 	{
 		ASStorageDstBuffer buf ; 
 		buf.offset = 0 ; 
 		buf.buffer = buffer ;
 	  	
-		return fetch_data_int( storage, id, &buf, offset, buf_size, bitmap_value, card8_card32_cpy );
+		return fetch_data_int( storage, id, &buf, offset, buf_size, bitmap_value, card8_card32_cpy, original_size );
 	}
 	return 0 ;	
 }
@@ -1597,6 +1614,7 @@ threshold_stored_data(ASStorage *storage, ASStorageID id, unsigned int *runs, in
 	if( storage != NULL && id != 0 )
 	{
 		ASStorageDstBuffer buf ; 
+		int dumm = 0 ;
 		buf.offset = 0 ; 
 		buf.buffer = runs ;
 
@@ -1607,7 +1625,7 @@ threshold_stored_data(ASStorage *storage, ASStorageID id, unsigned int *runs, in
 #ifdef DEBUG_THRESHOLD	  
 		fprintf( stderr, __FUNCTION__ ": id = 0x%lX, width = %d, threshold = %d\n", id, width, threshold );
 #endif
-		if( fetch_data_int( storage, id, &buf, 0, width, threshold, card8_threshold ) > 0 ) 
+		if( fetch_data_int( storage, id, &buf, 0, width, threshold, card8_threshold, &dumm) > 0 ) 
 		{
 			if( buf.start >= 0 && buf.end >= buf.start )
 			{
@@ -1803,7 +1821,7 @@ dup_data(ASStorage *storage, ASStorageID id)
 static int StorageTestKinds[STORAGE_TEST_KINDS][2] = 
 {
 	{100, 1 },
-	{4096, 40000 },
+	{4096, 10000 },
 	{128*1024, 128 },
 	{256*1024, 32 },
 	{512*1024, 16 },
@@ -1812,7 +1830,7 @@ static int StorageTestKinds[STORAGE_TEST_KINDS][2] =
 
 CARD8 Buffer[1024*1024] ;
 /* #define STORAGE_TEST_COUNT  1 */
-#define STORAGE_TEST_COUNT  8+16+32+128+40000+1 
+#define STORAGE_TEST_COUNT  8+16+32+128+10000+1 
 typedef struct ASStorageTest {
 	int size ;
 	CARD8 *data;
@@ -1988,6 +2006,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 	int i, kind, k;
 	int min_size, max_size ;
 	int test_count ;
+	START_TIME(started);
 
 	UsedMemory = 0 ;
 	UncompressedSize = 0 ;
@@ -2001,6 +2020,35 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 							else fprintf(stderr, "success.\n");}while(0)
 	TEST_EVAL( storage != NULL ); 
 	
+#ifdef DO_CLOCKING	
+	fprintf(stderr, "Testing speed for flags 0x%lX...", test_flags);
+	{
+#define SPEED_SIZE 1024*1024*50		
+		CARD8 *speed_buffer ;
+		int i, id ;
+		static CARD32 rnd32_seed = 345824357;
+
+		speed_buffer = safemalloc( SPEED_SIZE );
+
+		for( i = 0 ; i < SPEED_SIZE ; ++i ) 
+			speed_buffer[i] = (MY_RND32())&0x00FF ;
+		
+		{
+			START_TIME(started2);
+			id = store_data( storage, &speed_buffer[0], SPEED_SIZE, test_flags, 0 );
+			SHOW_TIME("RLE compression speed ", started2);
+		}
+		{
+			START_TIME(started3);
+			fetch_data(storage, id, &speed_buffer[0], 0, SPEED_SIZE, 0, NULL);
+			SHOW_TIME("RLE de-compression speed ", started3);
+			forget_data(storage, id );
+		}
+		free( speed_buffer );
+	}	 
+
+#endif
+	
 	fprintf(stderr, "Testing store_data for data %p size = %d, and flags 0x%lX...", NULL, 0,
 			test_flags);
 	id = store_data( storage, NULL, 0, test_flags, 0 );
@@ -2031,6 +2079,8 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 
 	fprintf( stderr, "%d :memory used %d #####################################################\n", __LINE__, UsedMemory );
 	fprintf( stderr, "%d :compressed_size = %d, uncompressed_size = %d, ratio = %d %% ###########\n", __LINE__, CompressedSize, UncompressedSize, (UncompressedSize<100)?0:(CompressedSize/(UncompressedSize/100)) );
+	SHOW_TIME("Pass 1", started);
+
 	if( interactive )
 	   fgetc(stdin);
 	for( i = 0 ; i < all_test_count ; ++i ) 
@@ -2038,7 +2088,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 		int size ;
 		int res ;
 		fprintf(stderr, "Testing fetch_data for id %lX size = %d ...", Tests[i].id, Tests[i].size);
-		size = fetch_data(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size, 0);
+		size = fetch_data(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size, 0, NULL);
 		TEST_EVAL( size == Tests[i].size ); 
 		
 		fprintf(stderr, "Testing fetched data integrity ...");
@@ -2052,7 +2102,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 			int size ;
 			int res ;
 			fprintf(stderr, "Testing fetch_data32 for id %lX size = %d ...", Tests[i].id, Tests[i].size);
-			size = fetch_data32(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size/4, 0);
+			size = fetch_data32(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size/4, 0, NULL);
 			TEST_EVAL( size == Tests[i].size/4 ); 
 		
 			fprintf(stderr, "Testing fetched data integrity ...");
@@ -2062,6 +2112,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 	}
 
 	fprintf( stderr, "%d :memory used %d #####################################################\n", __LINE__, UsedMemory );
+	SHOW_TIME("Pass 2", started);
 	if( interactive )
 	   fgetc(stdin);
 	for( i = 0 ; i < all_test_count ; ++i ) 
@@ -2072,7 +2123,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 			continue;
 		fprintf(stderr, "%d: Testing forget_data for id %lX size = %d ...\n", __LINE__, Tests[i].id, Tests[i].size);
 		forget_data(storage, Tests[i].id);
-		size = fetch_data(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size, 0 );
+		size = fetch_data(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size, 0, NULL );
 		TEST_EVAL( size != Tests[i].size ); 
 		Tests[i].id = 0;
 #ifndef DEBUG_ALLOCS
@@ -2084,6 +2135,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 		Tests[i].size = 0 ;
 	}	 
 	fprintf( stderr, "%d :memory used %d #####################################################\n", __LINE__, UsedMemory );
+	SHOW_TIME("Pass 3", started);
 	if( interactive )
 	   fgetc(stdin);
 	kind = 0 ; 
@@ -2110,6 +2162,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 		}		   
 	}	 
 	fprintf( stderr, "%d :memory used %d #####################################################\n", __LINE__, UsedMemory );
+	SHOW_TIME("Pass 4", started);
 	if( interactive )
 	   fgetc(stdin);
 	for( i = 0 ; i < all_test_count ; ++i ) 
@@ -2117,7 +2170,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 		int size ;
 		int res ;
 		fprintf(stderr, "Testing fetch_data for id %lX size = %d ...", Tests[i].id, Tests[i].size);
-		size = fetch_data(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size, 0);
+		size = fetch_data(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size, 0, NULL);
 		TEST_EVAL( size == Tests[i].size ); 
 		
 		fprintf(stderr, "Testing fetched data integrity ...");
@@ -2126,6 +2179,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 	}	 
 
 	fprintf( stderr, "%d :memory used %d #####################################################\n", __LINE__, UsedMemory );
+	SHOW_TIME("Pass 5", started);
 	if( interactive )
 	   fgetc(stdin);
 	for( i = 0 ; i < all_test_count ; ++i ) 
@@ -2136,7 +2190,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 			continue;
 		fprintf(stderr, "%d: Testing forget_data for id %lX size = %d ...\n", __LINE__, Tests[i].id, Tests[i].size);
 		forget_data(storage, Tests[i].id);
-		size = fetch_data(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size, 0);
+		size = fetch_data(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size, 0, NULL);
 		TEST_EVAL( size != Tests[i].size ); 
 		Tests[i].id = 0;
 #ifndef DEBUG_ALLOCS
@@ -2149,6 +2203,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 	}	 
 
 	fprintf( stderr, "%d :memory used %d #####################################################\n", __LINE__, UsedMemory );
+	SHOW_TIME("Pass 6", started);
 	if( interactive )
 		fgetc(stdin);
 	for( k = 0 ; k < 50 ; ++k )
@@ -2179,7 +2234,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 			Tests[i].size = Tests[k].size ;
 			Tests[i].data = Tests[k].data ;
 			Tests[i].linked = True ;
-			size = fetch_data(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size, 0);
+			size = fetch_data(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size, 0, NULL);
 			TEST_EVAL( size == Tests[i].size ); 
 		
 			fprintf(stderr, "Testing dupped data integrity ...\n");
@@ -2199,7 +2254,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 				continue;
 			fprintf(stderr, "%d: Testing forget_data for id %lX size = %d ...\n", __LINE__, Tests[i].id, Tests[i].size);
 			forget_data(storage, Tests[i].id);
-			size = fetch_data(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size, 0);
+			size = fetch_data(storage, Tests[i].id, &(Buffer[0]), 0, Tests[i].size, 0, NULL);
 			TEST_EVAL( size != Tests[i].size ); 
 			Tests[i].id = 0;
 			if( !Tests[i].linked ) 
@@ -2230,6 +2285,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 		}	 
 	}	
 	fprintf( stderr, "%d :memory used %d #####################################################\n", __LINE__, UsedMemory );
+	SHOW_TIME("Pass 7", started);
 	if( interactive )
 	   fgetc(stdin);
 	kind = 0 ; 
@@ -2257,7 +2313,7 @@ test_asstorage(Bool interactive, int all_test_count, ASFlagType test_flags )
 	}	 
 	fprintf( stderr, "%d :memory used %d #####################################################\n", __LINE__, UsedMemory );
 	fprintf( stderr, "%d :compressed_size = %d, uncompressed_size = %d, ratio = %d %% ###########\n", __LINE__, CompressedSize, UncompressedSize, (UncompressedSize<100)?0:(CompressedSize/(UncompressedSize/100)) );
-
+	SHOW_TIME("", started);
 	fprintf(stderr, "Testing storage destruction ...");
 	destroy_asstorage(&storage);
 	TEST_EVAL( storage == NULL ); 
@@ -2336,7 +2392,7 @@ int main(int argc, char **argv )
 		imdec = start_image_decoding(NULL, im, SCL_DO_ALL, 0, 0, im->width, im->height, NULL);
 		fprintf( stderr, "imdec = %p\n", imdec );
 	}
-	
+	fprintf(stderr, "running tests ( res = %d ) ...\n", res );	
 	if( res == 0 )
 		res = test_asstorage(interactive, test_count, 0);
 #if 0	  
