@@ -522,7 +522,11 @@ free_storage2quoted_text (FreeStorageElem * stored, int *ppos)
 
 	if (*ptr != '"')
 		ptr = find_doublequotes (ptr);
-	if (ptr)
+	if( ptr == NULL && get_flags( stored->term->flags, TF_QUOTES_OPTIONAL) )
+	{
+	 	result = mystrdup(stored->argv[*ppos]);		
+		(*ppos)++;
+	}else if (ptr)
 	{
 		ptr++;
 		if ((tail = find_doublequotes (ptr)) == NULL)
@@ -537,7 +541,7 @@ free_storage2quoted_text (FreeStorageElem * stored, int *ppos)
 				strcpy (ptr - 1, ptr);
 		(*ppos)++;
 	} else
-		show_error ("bad quoted string [%s] for option [%s]. Ignoring!",ptr, stored->term->keyword);
+		show_error ("bad quoted string [%s] (ppos == %d) for option [%s]. Ignoring!",ptr?ptr:"(null)", *ppos, stored->term->keyword);
 
 	return result;
 }
