@@ -679,8 +679,8 @@ DispatchEvent (ASEvent * event)
         case ReparentNotify :
             if( event->x.xreparent.parent == Scr.Root )
             {
-                sleep_a_millisec( 100 );
-                XMoveResizeWindow( dpy, event->x.xreparent.window, -1000, -1000, 1, 1 );
+                //sleep_a_millisec( 100 );
+                //XMoveResizeWindow( dpy, event->x.xreparent.window, -10000, -10000, 1, 1 );
             }
             break ;
 	    case PropertyNotify:
@@ -718,14 +718,28 @@ create_wharf_folder_canvas(ASWharfFolder *aswf)
     static XSetWindowAttributes attr ;
     Window w ;
     ASCanvas *pc = NULL;
-    unsigned long mask = CWEventMask|CWBackPixmap ;
+    unsigned long mask = CWEventMask|CWBackPixel ; /*map ; */
 
     attr.event_mask = WHARF_FOLDER_EVENT_MASK ;
     attr.background_pixel = Scr.asv->black_pixel ;
     attr.background_pixmap = ParentRelative ;
     if( Scr.asv->visual_info.visual != DefaultVisual( dpy, DefaultScreen(dpy)) )
         mask |= CWBackPixel ;
-    w = create_visual_window( Scr.asv, Scr.Root, 0, 0, 1, 1, 0, InputOutput, mask, &attr );
+    w = create_visual_window( Scr.asv, Scr.Root, 0, 0, 2, 2, 0, InputOutput, mask, &attr );
+
+#ifdef SHAPE
+	if( get_flags(Config->flags, WHARF_ANIMATE ) )
+	{
+		XRectangle rect ;
+		rect.x = rect.y = 0 ;
+		rect.width = rect.height = 1 ;
+		XShapeCombineRectangles ( dpy, w, ShapeBounding, 0, 0, &rect, 1, ShapeSet, Unsorted);
+		aswf->animate_from_w = 1;
+		aswf->animate_from_h = 1;
+		set_flags(aswf->flags,ASW_AnimationPending );
+	}
+#endif
+
     register_object( w, (ASMagic*)aswf );
     pc = create_ascanvas_container(w);
     LOCAL_DEBUG_OUT("folder canvas %p created for window %lX", pc, w );
@@ -741,7 +755,8 @@ create_wharf_button_canvas(ASWharfButton *aswb, ASCanvas *parent)
     ASCanvas *canvas;
 
     attr.event_mask = WHARF_BUTTON_EVENT_MASK ;
-    w = create_visual_window( Scr.asv, parent->w, -1, -1, 1, 1, 0, InputOutput, CWEventMask, &attr );
+	attr.background_pixel = Scr.asv->black_pixel ;
+    w = create_visual_window( Scr.asv, parent->w, -1, -1, 1, 1, 0, InputOutput, CWEventMask|CWBackPixel, &attr );
     register_object( w, (ASMagic*)aswb );
     canvas = create_ascanvas(w);
 
@@ -1095,14 +1110,17 @@ update_wharf_folder_shape( ASWharfFolder *aswf )
 		while ( --i >= 0 )
         {
             register ASWharfButton *aswb = &(aswf->buttons[i]);
-			LOCAL_DEBUG_OUT( "Adding shape of the button %d with geometry %dx%d%+d%+d, and geometry inside folder %dx%d%+d%+d", 
-							 i, aswb->canvas->width, aswb->canvas->height, aswb->canvas->root_x, aswb->canvas->root_y,
+			LOCAL_DEBUG_OUT( "Adding shape of the button %d (%p) with geometry %dx%d%+d%+d, and geometry inside folder %dx%d%+d%+d", 
+							 i, aswb, aswb->canvas->width, aswb->canvas->height, aswb->canvas->root_x, aswb->canvas->root_y,
 							 aswb->folder_width, aswb->folder_height, aswb->folder_x, aswb->folder_y );
             if( aswb->canvas->width == aswb->folder_width && aswb->canvas->height == aswb->folder_height )
                 if( combine_canvas_shape_at (aswf->canvas, aswb->canvas, aswb->folder_x, aswb->folder_y ) )
                     ++set;
             if( aswb->swallowed )
             {
+				refresh_container_shape( aswb->swallowed->current );
+				combine_canvas_shape_at (aswb->canvas, aswb->swallowed->current, 0, 0 );
+				update_canvas_display_mask (aswb->canvas, True);
                 if( combine_canvas_shape_at (aswf->canvas, aswb->swallowed->current, aswb->folder_x, aswb->folder_y ) )
                     ++set;
             }
@@ -1110,28 +1128,37 @@ update_wharf_folder_shape( ASWharfFolder *aswf )
 		if( get_flags( aswf->flags, ASW_UseBoundary) )
 		{
 			XRectangle sr ;
+			int do_subtract = 0 ;
 			sr = aswf->boundary ;
-/*			fprintf( stderr, "%s: boundary = %dx%d%+d%+d, canvas = %dx%d\n", 
-						__FUNCTION__, sr.width, sr.height, sr.x, sr.y, aswf->canvas->width, aswf->canvas->height ); */
+			LOCAL_DEBUG_OUT( "boundary = %dx%d%+d%+d, canvas = %dx%d\n", 
+						     sr.width, sr.height, sr.x, sr.y, aswf->canvas->width, aswf->canvas->height ); 
 			
-			if( sr.x > 0 )
-			{	 
-				sr.width = sr.x ;
-				sr.x = 0 ;
-			}else if( sr.width < aswf->canvas->width ) 
-			{
-				sr.x = sr.width ;
-				sr.width = aswf->canvas->width - sr.width ;
+			if( sr.width < aswf->canvas->width ) 
+			{	
+				if( sr.x > 0 )
+				{	 
+					sr.width = sr.x ;
+					sr.x = 0 ;
+				}else 
+				{
+					sr.x = sr.width ;
+					sr.width = aswf->canvas->width - sr.width ;
+				}
+				++do_subtract ;
 			}
 			
-			if( sr.y > 0 ) 
+			if( sr.height < aswf->canvas->height ) 
 			{	
-				sr.height = sr.y ;
-				sr.y = 0 ;
-			}else if( sr.height < aswf->canvas->height ) 
-			{
-				sr.y = sr.height ;
-				sr.height = aswf->canvas->height - sr.height ;
+				if( sr.y > 0 ) 
+				{	
+					sr.height = sr.y ;
+					sr.y = 0 ;
+				}else 
+				{
+					sr.y = sr.height ;
+					sr.height = aswf->canvas->height - sr.height ;
+				}
+				++do_subtract ;
 			}
 #if 0
 			fprintf( stderr, "%s: substr_boundary = %dx%d%+d%+d canvas = %dx%d%+d%+d\n", 
@@ -1139,10 +1166,12 @@ update_wharf_folder_shape( ASWharfFolder *aswf )
 					 aswf->canvas->width, aswf->canvas->height, aswf->canvas->root_x, aswf->canvas->root_y );
 			fprintf( stderr, "shape = %p, used = %d\n", aswf->canvas->shape, PVECTOR_USED(aswf->canvas->shape) );
 #endif			
-			if( sr.width > 0 && sr.height > 0 ) 
+			if( sr.width > 0 && sr.height > 0 && do_subtract > 0 ) 
+			{	
 				subtract_shape_rectangle( aswf->canvas->shape, &sr, 1, 0, 0, aswf->canvas->width, aswf->canvas->height );
+			}
 		}	 
-
+		
 		update_canvas_display_mask (aswf->canvas, True);
 
         if( set > 0 )
@@ -1184,6 +1213,7 @@ change_button_focus(ASWharfButton *aswb, Bool focused )
 	set_astbar_focused( aswb->bar, NULL, focused );			   
 	render_wharf_button( aswb );
 	update_canvas_display( aswb->canvas );
+	update_wharf_folder_shape( aswb->parent );
 
 	if( focused ) 
 		WharfState.focused_button = aswb ;
@@ -1215,15 +1245,12 @@ update_wharf_folder_styles( ASWharfFolder *aswf, Bool force )
 
 void
 map_wharf_folder( ASWharfFolder *aswf,
-                  int x, int y, unsigned int width, unsigned int height,
                   int gravity )
 {
     XSizeHints      shints;
 	ExtendedWMHints extwm_hints ;
     ASFlagType protocols = 0;
 
-    moveresize_canvas( aswf->canvas, x, y, width, height );
-
 	set_folder_name( aswf, False );
 
     shints.flags = USSize|PWinGravity;
@@ -1247,12 +1274,23 @@ map_wharf_folder( ASWharfFolder *aswf,
 
     set_client_hints( aswf->canvas->w, NULL, &shints, AS_DoesWmDeleteWindow, &extwm_hints );
 
+//	ASSync(False);
+//	sleep_a_millisec (10);
 	/* showing window to let user see that we are doing something */
     map_canvas_window(aswf->canvas, True);
-    LOCAL_DEBUG_OUT( "mapping main window at %ux%u%+d%+d", width, height,  x, y );
+#ifdef SHAPE
+	if( get_flags(Config->flags, WHARF_ANIMATE ) )
+	{	
+		XShapeCombineRectangles ( dpy, aswf->canvas->w, ShapeBounding, 0, 0, &(aswf->boundary), 1, ShapeSet, Unsorted);
+		aswf->animate_from_w = 1;
+		aswf->animate_from_h = 1;
+		set_flags(aswf->flags,ASW_AnimationPending );
+	}
+#endif
+    LOCAL_DEBUG_OUT( "mapping folder window for folder %p", aswf );
     /* final cleanup */
-    ASSync( False );
-    sleep_a_millisec (10);                                 /* we have to give AS a chance to spot us */
+//    ASSync( False );
+//    sleep_a_millisec (10);                                 /* we have to give AS a chance to spot us */
 }
 
 void
@@ -1424,6 +1462,31 @@ place_wharf_buttons( ASWharfFolder *aswf, int *total_width_return, int *total_he
         clear_flags( aswf->flags, ASW_NeedsShaping );
 }
 
+
+void
+clamp_animation_rect( ASWharfFolder *aswf, int from_width, int from_height, int to_width, int to_height, XRectangle *rect )
+{
+	if( get_flags( aswf->flags, ASW_Vertical ) )
+	{	
+		if( aswf->gravity == SouthWestGravity || aswf->gravity == SouthEastGravity )
+			rect->y = max(to_height,from_height) - (short)rect->height ;
+		if( rect->y < 0 ) 
+		{	
+			rect->y = 0 ; 
+			rect->height = to_height ;
+		}
+	}else 
+	{	
+		if( aswf->gravity == NorthEastGravity || aswf->gravity == SouthEastGravity )
+			rect->x = max(to_width,from_width) - (short)rect->width ;
+		if( rect->x < 0 ) 
+		{	
+			rect->x = 0 ; 
+			rect->width = to_width ;
+		}
+	}
+}	 
+
 void
 animate_wharf_loop(ASWharfFolder *aswf, int from_width, int from_height, int to_width, int to_height )	
 {
@@ -1431,27 +1494,20 @@ animate_wharf_loop(ASWharfFolder *aswf, int from_width, int from_height, int to_
 	XRectangle rect ;
 
 	steps = get_flags( Config->set_flags, WHARF_ANIMATE_STEPS )?Config->animate_steps:12;
-
+	LOCAL_DEBUG_OUT( "steps = %d", steps );
 	for( i = 0 ; i < steps ; ++i )
 	{
 		rect.x = rect.y = 0 ;		
 		rect.width = get_flags( aswf->flags, ASW_Vertical )?to_width:from_width+((to_width-from_width)/steps)*(i+1) ;
 		rect.height = get_flags( aswf->flags, ASW_Vertical )?from_height+((to_height-from_height)/steps)*(i+1):to_height ;
+
+		clamp_animation_rect( aswf, from_width, from_height, to_width, to_height, &rect );
 		
-		if( get_flags( aswf->flags, ASW_Vertical ) )
-		{	
-			if( aswf->gravity == SouthWestGravity || aswf->gravity == SouthEastGravity )
-				rect.y = max(to_height,from_height) - rect.height ;
-		}else 
-		{	
-			if( aswf->gravity == NorthEastGravity || aswf->gravity == SouthEastGravity )
-				rect.x = max(to_width,from_width) - rect.width ;
-		}
-		
-		/*fprintf( stderr, "%s: boundary = %dx%d%+d%+d, canvas = %dx%d\n", 
-						__FUNCTION__, rect.width, rect.height, rect.x, rect.y, aswf->canvas->width, aswf->canvas->height ); */
+		LOCAL_DEBUG_OUT("boundary = %dx%d%+d%+d, canvas = %dx%d\n", 
+						 rect.width, rect.height, rect.x, rect.y, 
+						 aswf->canvas->width, aswf->canvas->height );
 		if( rect.x+rect.width > aswf->canvas->width ||
-			rect.x+rect.height > aswf->canvas->height ) 
+			rect.y+rect.height > aswf->canvas->height ) 
 		{
 			return;			
 		} 
@@ -1470,18 +1526,10 @@ animate_wharf_loop(ASWharfFolder *aswf, int from_width, int from_height, int to_
 	rect.x = rect.y = 0 ;
 	rect.width = to_width ;
 	rect.height = to_height ;
-	if( get_flags( aswf->flags, ASW_Vertical ) )
-	{	
-		if( aswf->gravity == SouthWestGravity || aswf->gravity == SouthEastGravity )
-			rect.y = max(to_height,from_height) - rect.height ;
-	}else 
-	{	
-		if( aswf->gravity == NorthEastGravity || aswf->gravity == SouthEastGravity )
-			rect.x = max(to_width,from_width) - rect.width ;
-	}
+	clamp_animation_rect( aswf, from_width, from_height, to_width, to_height, &rect );
 	
-	/* fprintf( stderr, "%s: boundary = %dx%d%+d%+d, canvas = %dx%d\n", 
-						__FUNCTION__, rect.width, rect.height, rect.x, rect.y, aswf->canvas->width, aswf->canvas->height ); */
+	LOCAL_DEBUG_OUT("boundary = %dx%d%+d%+d, canvas = %dx%d\n", 
+					rect.width, rect.height, rect.x, rect.y, aswf->canvas->width, aswf->canvas->height );
 
 	aswf->boundary = rect ;
 	update_wharf_folder_shape( aswf );
@@ -1657,50 +1705,64 @@ display_wharf_folder( ASWharfFolder *aswf, int left, int top, int right, int bot
 	LOCAL_DEBUG_OUT("corrected  pos(%+d%+d)", x, y );
     LOCAL_DEBUG_OUT( "flags 0x%lX, reverse_order = %d", aswf->flags, get_flags( aswf->flags, ASW_ReverseOrder)?aswf->buttons_num-1:-1 );
 
+	moveresize_canvas( aswf->canvas, x, y, width, height );
 	set_wharf_clip_area( aswf, left, top );
-
-    if( get_flags(Config->flags, WHARF_ANIMATE ) )
+	
+	aswf->animate_to_w = width;
+	aswf->animate_to_h = height;
+    
+	if( get_flags(Config->flags, WHARF_ANIMATE ) )
     {
 		set_flags(aswf->flags,ASW_UseBoundary|ASW_AnimationPending );
-		aswf->animate_from_w = aswf->canvas->width; 
-		aswf->animate_from_h = aswf->canvas->height;
-		aswf->animate_to_w = width;
-		aswf->animate_to_h = height;
+		aswf->animate_from_w = get_flags( aswf->flags, ASW_Vertical )?aswf->canvas->width:0; 
+		aswf->animate_from_h = get_flags( aswf->flags, ASW_Vertical )?0:aswf->canvas->height;
 		aswf->boundary.x = aswf->boundary.y = 0 ; 
 		if( get_flags( aswf->flags, ASW_Vertical ) )
 		{		
-			aswf->boundary.height = (aswf->gravity == NorthEastGravity || aswf->gravity == NorthWestGravity )?aswf->canvas->height:1 ;
-			aswf->boundary.width = (aswf->boundary.height > 1)?aswf->canvas->width:1 ;
+			if( south )
+				aswf->boundary.y = height-1 ; 
+			aswf->boundary.height = 1 ;
+			aswf->boundary.width = 1 ;
 		}else
 		{
-			aswf->boundary.width = (aswf->gravity == NorthWestGravity || aswf->gravity == SouthWestGravity )?aswf->canvas->width:1 ;
-			aswf->boundary.height = (aswf->boundary.width > 1)?aswf->canvas->height:1 ;
+			if( east )
+				aswf->boundary.x = width-1 ; 
+			aswf->boundary.height = 1 ;
+			aswf->boundary.width = 1 ;
+
+/* 			aswf->boundary.width = (aswf->gravity == NorthWestGravity || aswf->gravity == SouthWestGravity )?aswf->canvas->width:1 ;
+			aswf->boundary.height = (aswf->boundary.width == 1)?aswf->canvas->height:1 ;
+ */ 
 		}	 
 #ifdef SHAPE		
+		LOCAL_DEBUG_OUT("boundary pos(%dx%d%+d%+d) shaping window %lX", aswf->boundary.width, aswf->boundary.height, aswf->boundary.x, aswf->boundary.y, aswf->canvas->w );
 		/* fprintf( stderr, "setting boundary to 1x1\n" );  */
-	    XShapeCombineRectangles ( dpy, aswf->canvas->w, ShapeBounding,
-                                  0, 0, &(aswf->boundary), 1, ShapeSet, Unsorted);
+	    XShapeCombineRectangles ( dpy, aswf->canvas->w, ShapeBounding, 0, 0, &(aswf->boundary), 1, ShapeSet, Unsorted);
 #endif
     }
     
-	map_wharf_folder( aswf, x, y, width, height, east?(south?SouthEastGravity:NorthEastGravity):
-                                                      (south?SouthWestGravity:NorthWestGravity) );
+	map_wharf_folder( aswf, east?(south?SouthEastGravity:NorthEastGravity):
+                                 (south?SouthWestGravity:NorthWestGravity) );
 
 	set_flags( aswf->flags, ASW_Mapped );
     clear_flags( aswf->flags, ASW_Withdrawn );
 	ASSync(False);
+
+	if( aswf->canvas->width == width && aswf->canvas->height == height && get_flags(Config->flags, WHARF_ANIMATE ))
+	{
+		animate_wharf_loop( aswf, aswf->animate_from_w, aswf->animate_from_h, aswf->animate_to_w, aswf->animate_to_h );
+		clear_flags(aswf->flags,ASW_UseBoundary|ASW_AnimationPending );
+	}	 
+
     return True;
 }
 
+static inline void unmap_wharf_folder( ASWharfFolder *aswf );
+
 static inline void
-unmap_wharf_folder( ASWharfFolder *aswf )
+unmap_wharf_subfolders( ASWharfFolder *aswf )
 {
     int i = aswf->buttons_num;
-LOCAL_DEBUG_OUT( "unmapping canvas %p at %dx%d%+d%+d", aswf->canvas, aswf->canvas->width, aswf->canvas->height, aswf->canvas->root_x, aswf->canvas->root_y );
-    unmap_canvas_window( aswf->canvas );
-    /*moveresize_canvas( aswf->canvas, -1000, -1000, 1, 1 ); to make sure we get ConfigureNotify next time we map the folder again */
-    clear_flags( aswf->flags, ASW_Mapped );
-
     while ( --i >= 0 )
     {
         if( aswf->buttons[i].folder &&
@@ -1709,6 +1771,20 @@ LOCAL_DEBUG_OUT( "unmapping canvas %p at %dx%d%+d%+d", aswf->canvas, aswf->canva
     }
 }
 
+static inline void
+unmap_wharf_folder( ASWharfFolder *aswf )
+{
+LOCAL_DEBUG_OUT( "unmapping canvas %p at %dx%d%+d%+d", aswf->canvas, aswf->canvas->width, aswf->canvas->height, aswf->canvas->root_x, aswf->canvas->root_y );
+    unmap_canvas_window( aswf->canvas );
+    /*moveresize_canvas( aswf->canvas, -1000, -1000, 1, 1 ); to make sure we get ConfigureNotify next time we map the folder again */
+    clear_flags( aswf->flags, ASW_Mapped );
+
+	unmap_wharf_subfolders( aswf );
+}
+
+static inline void
+withdraw_wharf_subfolders( ASWharfFolder *aswf );
+
 void
 withdraw_wharf_folder( ASWharfFolder *aswf )
 {
@@ -1719,28 +1795,41 @@ LOCAL_DEBUG_OUT( "folder->flags(%lX)", aswf->flags );
     if( !get_flags( aswf->flags, ASW_Mapped ) )
         return ;
 
+	withdraw_wharf_subfolders( aswf );
+	ASSync(False);
+
     if( get_flags(Config->flags, WHARF_ANIMATE ) )
     {
-        int i = aswf->buttons_num;
-        while ( --i >= 0 )
-        {
-            if( aswf->buttons[i].folder &&
-                get_flags( aswf->buttons[i].folder->flags, ASW_Mapped ) )
-            unmap_wharf_folder( aswf->buttons[i].folder );
-        }
-		
 		set_flags(aswf->flags,ASW_UseBoundary );
 		aswf->boundary.x = aswf->boundary.y = 0 ; 
 		aswf->boundary.width = aswf->canvas->width ;
 		aswf->boundary.height = aswf->canvas->height ;
 		
-		animate_wharf_loop(aswf, aswf->canvas->width, aswf->canvas->height, 1, 1 );
+		if( get_flags(aswf->flags,ASW_Vertical) )
+			animate_wharf_loop(aswf, aswf->canvas->width, aswf->canvas->height, aswf->canvas->width, 1 );
+		else
+			animate_wharf_loop(aswf, aswf->canvas->width, aswf->canvas->height, 1, aswf->canvas->height );
     }
 LOCAL_DEBUG_OUT( "unmapping folder %p", aswf );
     unmap_wharf_folder( aswf );
     ASSync( False );
+	sleep_a_millisec( 10 );
 }
 
+static inline void
+withdraw_wharf_subfolders( ASWharfFolder *aswf )
+{
+    int i = aswf->buttons_num;
+    while ( --i >= 0 )
+    {
+        if( aswf->buttons[i].folder &&
+            get_flags( aswf->buttons[i].folder->flags, ASW_Mapped ) )
+            withdraw_wharf_folder( aswf->buttons[i].folder );
+    }
+}
+
+
+
 Bool display_main_folder()
 {
     int left = Config->geometry.x;
@@ -1864,7 +1953,9 @@ void
 update_wharf_folder_size( ASWharfFolder *aswf)
 {
     unsigned int total_width = 1, total_height = 1;
-    place_wharf_buttons( aswf, &total_width, &total_height );
+    if( !get_flags( aswf->flags, ASW_Mapped) )
+		return;		
+	place_wharf_buttons( aswf, &total_width, &total_height );
 
     if( total_width != 0 && total_height != 0 )
     {
@@ -1994,8 +2085,7 @@ check_swallow_window( ASWindowData *wd )
                        swidth, sheight );
     map_canvas_window( aswb->swallowed->current, True );
     send_swallowed_configure_notify(aswb);
-
-    update_wharf_folder_size( aswf );
+	update_wharf_folder_size( aswf );
 }
 
 
@@ -2351,45 +2441,56 @@ void on_wharf_moveresize( ASEvent *event )
     {
         ASWharfFolder *aswf = (ASWharfFolder*)obj;
         ASFlagType changes = handle_canvas_config (aswf->canvas );
-		LOCAL_DEBUG_OUT("Handling folder resize for button %p", aswf );
+		LOCAL_DEBUG_OUT("Handling folder resize for folder %p, mapped = %lX", aswf, get_flags( aswf->flags, ASW_Mapped ) );
         if( aswf->animation_steps == 0 && get_flags( aswf->flags, ASW_Mapped ) && aswf->animation_dir < 0 )
         {
             unmap_wharf_folder( aswf );
         }else if( changes != 0 )
         {
-LOCAL_DEBUG_OUT("animation_steps = %d", aswf->animation_steps );
             int i = aswf->buttons_num ;
+			Bool withdrawn = False ;
+LOCAL_DEBUG_OUT("animation_steps = %d", aswf->animation_steps );
 
-			if( !get_flags( aswf->flags, ASW_Withdrawn ) )
+			withdrawn = (aswf->canvas->width == 1 || aswf->canvas->height == 1 ||
+						 (aswf->canvas->root_x == -10000 && aswf->canvas->root_y == -10000) );
+
+#ifdef SHAPE
+            if( get_flags( changes, CANVAS_RESIZED ) && get_flags(aswf->flags,ASW_AnimationPending ) )
+				XShapeCombineRectangles ( dpy, aswf->canvas->w, ShapeBounding, 0, 0, &(aswf->boundary), 1, ShapeSet, Unsorted);
+#endif
+
+			if( !withdrawn )
 			{	
-				set_wharf_clip_area( aswf, aswf->canvas->root_x, aswf->canvas->root_y );
-				while( --i >= 0 )
-    	            on_wharf_button_moveresize( &(aswf->buttons[i]), event );
-			}else if( aswf->withdrawn_button != NULL )
-				on_wharf_button_moveresize( aswf->withdrawn_button, event );
+				if( !get_flags( aswf->flags, ASW_Withdrawn ))
+				{	
+					set_wharf_clip_area( aswf, aswf->canvas->root_x, aswf->canvas->root_y );
+					while( --i >= 0 )
+    	            	on_wharf_button_moveresize( &(aswf->buttons[i]), event );
+				}else if( aswf->withdrawn_button != NULL )
+					on_wharf_button_moveresize( aswf->withdrawn_button, event );
+			}
 
 #if 1			   
-            if( get_flags( changes, CANVAS_RESIZED ) )
+            if( get_flags( changes, CANVAS_RESIZED ))
 			{
+				LOCAL_DEBUG_OUT("AnimationPending ? = %lX", get_flags(aswf->flags,ASW_AnimationPending));
 				if( get_flags(aswf->flags,ASW_AnimationPending ) )
 				{	
+					LOCAL_DEBUG_OUT("animate from = %dx%d, to = %dx%d", aswf->animate_from_w, aswf->animate_from_h, aswf->animate_to_w, aswf->animate_to_h);
 					animate_wharf_loop( aswf, aswf->animate_from_w, aswf->animate_from_h, aswf->animate_to_w, aswf->animate_to_h );
 					clear_flags(aswf->flags,ASW_UseBoundary|ASW_AnimationPending );
+				}else if( !withdrawn )
+				{
+					/* fprintf(stderr, "clearing or applying boundary\n");	  */
+					if( !update_wharf_folder_shape( aswf ) ) 
+					{	
+						/* fprintf(stderr, "forcing boundary\n");	  */
+						update_canvas_display_mask(aswf->canvas, True);
+					}
 				}
 			}
 #endif
 		
-			if( get_flags( changes, CANVAS_RESIZED ) )
-			{
-				/* fprintf(stderr, "clearing or applying boundary\n");	  */
-				if( !update_wharf_folder_shape( aswf ) ) 
-				{	
-					/* fprintf(stderr, "forcing boundary\n");	  */
-					update_canvas_display_mask(aswf->canvas, True);
-				}
-			}
-
-
         }
     }
 }
@@ -2467,7 +2568,7 @@ on_wharf_pressed( ASEvent *event )
 		if( get_flags( aswb->flags, ASW_Transient ) )
 			return ;
 		        
-		if( event->x.xbutton.button == Button3 && aswb->parent == WharfState.root_folder )
+		if( event->x.xbutton.button == Button3 && aswf == WharfState.root_folder )
 		{	
             if( (WITHDRAW_ON_EDGE(Config) && (&(aswf->buttons[0]) == aswb || &(aswf->buttons[aswf->buttons_num-1]) == aswb)) ||
                  WITHDRAW_ON_ANY(Config))
@@ -2486,8 +2587,9 @@ on_wharf_pressed( ASEvent *event )
 					if( Config->withdraw_style < WITHDRAW_ON_ANY_BUTTON_AND_SHOW )
                     	aswb = &(aswf->buttons[0]);
 
+					withdraw_wharf_subfolders( aswf );
 					/* update our name to withdrawn */	  					
-					set_folder_name( WharfState.root_folder, True );
+					set_folder_name( aswf, True );
 
                 	wwidth = aswb->desired_width ;
                 	wheight = aswb->desired_height ;
@@ -2496,18 +2598,17 @@ on_wharf_pressed( ASEvent *event )
                 	if( get_flags( Config->geometry.flags, YNegative ) )
                     	wy = Scr.MyDisplayHeight - wheight ;
 
-					set_flags(aswf->flags,ASW_UseBoundary );
-					animate_wharf_loop(aswf, aswf->canvas->width, aswf->canvas->height, wwidth, wheight );
-					clear_flags(aswf->flags,ASW_UseBoundary );
-
+					if( get_flags(Config->flags, WHARF_ANIMATE ) )
+					{	
+						set_flags(aswf->flags,ASW_UseBoundary );
+						animate_wharf_loop(aswf, aswf->canvas->width, aswf->canvas->height, wwidth, wheight );
+						clear_flags(aswf->flags,ASW_UseBoundary );
+					}
 
 					LOCAL_DEBUG_OUT( "withdrawing folder to %dx%d%+d%+d", wwidth, wheight, wx, wy );
                 	XRaiseWindow( dpy, aswb->canvas->w );
                 	while ( --i >= 0 )
                 	{
-                    	if( aswf->buttons[i].folder &&
-                        	get_flags( aswf->buttons[i].folder->flags, ASW_Mapped ) )
-                        	unmap_wharf_folder( aswf->buttons[i].folder );
                     	if( &(aswf->buttons[i]) != aswb && 
 							aswf->buttons[i].canvas->root_x == aswf->canvas->root_x && 
 							aswf->buttons[i].canvas->root_y == aswf->canvas->root_y)
