@@ -655,7 +655,8 @@ static Bool do_random_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry
             new_y = MY_RND32();
             new_y = (new_y % ( rects[selected].height - h ));
         }
-
+		LOCAL_DEBUG_OUT( "rect %dx%d%+d%+d, new_pos = %+d%+d", 
+						 rects[selected].width, rects[selected].height, rects[selected].x, rects[selected].y, new_x, new_y );
         apply_placement_result_asw( asw, XValue|YValue, rects[selected].x+new_x, rects[selected].y+new_y, 0, 0 );
         LOCAL_DEBUG_OUT( "success: status(%+d%+d), anchor(%+d,%+d)", asw->status->x, asw->status->y, asw->anchor.x, asw->anchor.y );
     }else
@@ -1048,6 +1049,9 @@ Bool place_aswindow( ASWindow *asw )
         aswbox = &(Scr.Feel.window_boxes[0]);
         while( --i >= 0 )
         {
+			LOCAL_DEBUG_OUT("window_box \"%s\": main_strategy = %d, backup_strategy = %d", 
+		   					aswbox[i].name, aswbox[i].main_strategy, aswbox[i].backup_strategy);
+			
             if( IsValidDesk(aswbox[i].desk) && aswbox[i].desk != asw->status->desktop )
                 continue;
             if( aswbox[i].min_layer > asw->status->layer || aswbox[i].max_layer < asw->status->layer )
