@@ -370,7 +370,8 @@ condense_tbar( ASTBarData *tbar, unsigned int max_size, unsigned int *off1, unsi
                         *off2 = *off1 ;
                     }    
                 }
-            }
+            }else 
+				condensed = max_size ;
         }else
         {
             condensed = calculate_astbar_width( tbar );                
@@ -385,7 +386,8 @@ condense_tbar( ASTBarData *tbar, unsigned int max_size, unsigned int *off1, unsi
                         *off2 = *off1 ;
                     }    
                 }
-            }
+            }else 
+				condensed = max_size ;
         }    
     }
 #endif
@@ -447,7 +449,7 @@ move_resize_frame_bars( ASWindow *asw, int side, ASOrientation *od, unsigned int
     /* side */
     if( longbar )
     {
-        if( title == NULL && corner_size1>0 && corner_size2 > 0 )                    /* we are in the sbar */
+        if( side != od->tbar_side && (corner_size1>0 || corner_size2 > 0) )                    /* we are in the sbar */
         {
             *(od->in_width) = longbar->width ;
             *(od->in_height) = longbar->height ;
@@ -763,6 +765,9 @@ LOCAL_DEBUG_OUT( "changes=0x%X", changes );
     ASSync(False);
 }
 
+int update_window_tbar_size( ASWindow *asw );
+
+
 void
 on_window_title_changed( ASWindow *asw, Bool update_display )
 {
@@ -774,9 +779,22 @@ on_window_title_changed( ASWindow *asw, Bool update_display )
         if( change_astbar_first_label( asw->tbar, asw->hints->names[0], asw->hints->names_encoding[0] ) )
             if( canvas && update_display )
             {
-                render_astbar( asw->tbar, canvas );
 				invalidate_canvas_save( canvas );
+				
+				if( canvas->shape ) 
+				{
+					XRectangle rect ;
+					rect.x = asw->tbar->win_x ;
+					rect.y = asw->tbar->win_y ;
+					rect.width = asw->tbar->width ;
+					rect.height = asw->tbar->height ;
+					subtract_shape_rectangle( canvas->shape, &rect, 1, 0, 0, canvas->width, canvas->height );
+				}
+				update_window_tbar_size( asw );
+                render_astbar( asw->tbar, canvas );
                 update_canvas_display( canvas );
+				//if( ASWIN_GET_FLAGS( asw, AS_ShapedDecor ) )
+					SetShape( asw, 0 );
             }
     }
     if( asw->icon_title )
@@ -844,6 +862,69 @@ on_window_hints_changed( ASWindow *asw )
 	destroy_hints (old_hints, True);
 }
 
+int
+update_window_tbar_size(	ASWindow *asw )
+{
+    int tbar_size = 0 ;
+	if( asw->tbar )
+    {    
+        unsigned int tbar_width = 0 ;
+        unsigned int tbar_height = 0 ;
+        int x_offset = 0, y_offset = 0 ;
+        if( ASWIN_HFLAGS(asw, AS_VerticalTitle) )
+        {
+            tbar_size = calculate_astbar_width( asw->tbar );
+            tbar_width = tbar_size ; 
+            tbar_height = asw->frame_canvas->height ;
+#ifdef SHAPE
+            if( get_flags( asw->frame_data->condense_titlebar, ALIGN_LEFT|ALIGN_RIGHT )  )
+            {
+                int condensed = calculate_astbar_height( asw->tbar );
+                if( condensed < tbar_height ) 
+                {    
+                    if( get_flags( asw->frame_data->condense_titlebar, ALIGN_LEFT ) )
+                    {    
+                        y_offset = tbar_height - condensed ;
+                        if( get_flags( asw->frame_data->condense_titlebar, ALIGN_RIGHT ) )
+                            y_offset /= 2 ;
+                    }
+                    tbar_height = condensed ;
+                }
+            }    
+#endif
+        }else
+        {
+            tbar_size = calculate_astbar_height( asw->tbar );
+            tbar_width = asw->frame_canvas->width ; 
+            tbar_height = tbar_size ;
+#ifdef SHAPE
+            if( get_flags( asw->frame_data->condense_titlebar, ALIGN_LEFT|ALIGN_RIGHT )  )
+            {
+                int condensed = calculate_astbar_width( asw->tbar );
+                if( condensed < tbar_width ) 
+                {    
+                    if( get_flags( asw->frame_data->condense_titlebar, ALIGN_RIGHT ) )
+                    {    
+                        x_offset = tbar_width - condensed ;
+                        if( get_flags( asw->frame_data->condense_titlebar, ALIGN_LEFT ) )
+                            x_offset /= 2 ;
+                    }
+                    tbar_width = condensed ;
+                }
+            }    
+#endif
+        }
+        /* we need that to set up tbar size : */
+        set_astbar_size( asw->tbar, tbar_width, tbar_height );
+        /* does not matter if we use frame canvas, since part's 
+            * canvas resizes at frame canvas origin anyway */
+        move_astbar( asw->tbar, asw->frame_canvas, x_offset, y_offset );
+
+    }
+	return tbar_size;
+}
+
+
 void
 on_window_status_changed( ASWindow *asw, Bool update_display, Bool reconfigured )
 {
@@ -900,16 +981,18 @@ LOCAL_DEBUG_OUT( "status geometry = %dx%d%+d%+d", asw->status->width, asw->statu
                                         unfocus_mystyle : asw->frame_data->frame_style_names[back_type];
 
         for( i = 0 ; i < FRAME_PARTS ; ++i )
-            if( asw->frame_bars[i] )
+		{
+			unsigned int real_part = od->frame_rotation[i];
+	        if( asw->frame_bars[real_part] )
             {
-                if( set_astbar_style( asw->frame_bars[i], BAR_STATE_UNFOCUSED, frame_unfocus_mystyle ) )
+                if( set_astbar_style( asw->frame_bars[real_part], BAR_STATE_UNFOCUSED, frame_unfocus_mystyle ) )
                     changed = True ;
-                if( set_astbar_hilite( asw->frame_bars[i], BAR_STATE_UNFOCUSED, frame_bevel[i] ) )
+                if( set_astbar_hilite( asw->frame_bars[real_part], BAR_STATE_UNFOCUSED, frame_bevel[i] ) )
                     changed = True ;
-                if( is_astbar_shaped( asw->frame_bars[i], -1 ) )
+                if( is_astbar_shaped( asw->frame_bars[real_part], -1 ) )
                     decor_shaped = True;
             }
-
+		}
         if( asw->tbar )
         {
             if( set_astbar_style( asw->tbar, BAR_STATE_UNFOCUSED, unfocus_mystyle ) )
@@ -933,61 +1016,10 @@ LOCAL_DEBUG_OUT( "status geometry = %dx%d%+d%+d", asw->status->width, asw->statu
         {/* now we need to update frame sizes in status */
             unsigned int *frame_size = &(asw->status->frame_size[0]) ;
             unsigned short tbar_size = 0;
-            if( asw->tbar )
-            {    
-                unsigned int tbar_width = 0 ;
-                unsigned int tbar_height = 0 ;
-                int x_offset = 0, y_offset = 0 ;
-                if( ASWIN_HFLAGS(asw, AS_VerticalTitle) )
-                {
-                    tbar_size = calculate_astbar_width( asw->tbar );
-                    tbar_width = tbar_size ; 
-                    tbar_height = asw->frame_canvas->height ;
-#ifdef SHAPE
-                    if( get_flags( asw->frame_data->condense_titlebar, ALIGN_LEFT|ALIGN_RIGHT )  )
-                    {
-                        int condensed = calculate_astbar_height( asw->tbar );
-                        if( condensed < tbar_height ) 
-                        {    
-                            if( get_flags( asw->frame_data->condense_titlebar, ALIGN_LEFT ) )
-                            {    
-                                y_offset = tbar_height - condensed ;
-                                if( get_flags( asw->frame_data->condense_titlebar, ALIGN_RIGHT ) )
-                                    y_offset /= 2 ;
-                            }
-                            tbar_height = condensed ;
-                        }
-                    }    
-#endif
-                }else
-                {
-                    tbar_size = calculate_astbar_height( asw->tbar );
-                    tbar_width = asw->frame_canvas->width ; 
-                    tbar_height = tbar_size ;
-#ifdef SHAPE
-                    if( get_flags( asw->frame_data->condense_titlebar, ALIGN_LEFT|ALIGN_RIGHT )  )
-                    {
-                        int condensed = calculate_astbar_width( asw->tbar );
-                        if( condensed < tbar_width ) 
-                        {    
-                            if( get_flags( asw->frame_data->condense_titlebar, ALIGN_RIGHT ) )
-                            {    
-                                x_offset = tbar_width - condensed ;
-                                if( get_flags( asw->frame_data->condense_titlebar, ALIGN_LEFT ) )
-                                    x_offset /= 2 ;
-                            }
-                            tbar_width = condensed ;
-                        }
-                    }    
-#endif
-                }
-                /* we need that to set up tbar size : */
-                set_astbar_size( asw->tbar, tbar_width, tbar_height );
-                /* does not matter if we use frame canvas, since part's 
-                 * canvas resizes at frame canvas origin anyway */
-                move_astbar( asw->tbar, asw->frame_canvas, x_offset, y_offset );
-
-            }
+			int bw = 0 ;
+			if( asw->hints && get_flags(asw->hints->flags, AS_Border)) 
+				bw = asw->hints->border_width ;
+			tbar_size = update_window_tbar_size( asw );
             for( i = 0 ; i < FRAME_SIDES ; ++i )
             {
                 if( asw->frame_bars[i] )
@@ -995,6 +1027,7 @@ LOCAL_DEBUG_OUT( "status geometry = %dx%d%+d%+d", asw->status->width, asw->statu
                                                     asw->frame_bars[i]->height ;
                 else
                     frame_size[i] = 0;
+				 //frame_size[i] += bw ;
             }
 			if( tbar_size > 0 )
 			{
@@ -1063,55 +1096,52 @@ void
 on_window_hilite_changed( ASWindow *asw, Bool focused )
 {
     ASOrientation *od = get_orientation_data( asw );
+    int i;
 
 LOCAL_DEBUG_CALLER_OUT( "(%p,%s focused)", asw, focused?"":"not" );
     if( AS_ASSERT(asw) )
         return;
 
-    if(!ASWIN_GET_FLAGS(asw, AS_Iconic))
-    {
-        register int i = FRAME_PARTS;
-
-	    for( i = FRAME_SIDES; --i >= 0; )
+	for( i = FRAME_SIDES; --i >= 0; )
+	{
+    	ASCanvas *update_canvas = asw->frame_sides[i] ;
+		int k ;
+		if( swap_save_canvas( update_canvas ) )
+		{	
+			LOCAL_DEBUG_OUT( "canvas save swapped for side %d", i );
+			update_canvas = NULL ;
+		}
+#if 0				  
+		if( focused )
 		{
-    	    ASCanvas *update_canvas = asw->frame_sides[i] ;
-			int k ;
-			if( swap_save_canvas( update_canvas ) )
-			{	
-				LOCAL_DEBUG_OUT( "canvas save swapped for side %d", i );
+			save_canvas(update_canvas);
+		}else
+		{
+			if( restore_canvas(asw->frame_sides[i]) )
+			{
+				LOCAL_DEBUG_OUT( "canvas restored for side %d", i );
 				update_canvas = NULL ;
 			}
-#if 0				  
-			if( focused )
-			{
-				save_canvas(update_canvas);
-			}else
-			{
-				if( restore_canvas(asw->frame_sides[i]) )
-				{
-					LOCAL_DEBUG_OUT( "canvas restored for side %d", i );
-					update_canvas = NULL ;
-				}
-			}
+		}
 #endif
 
-			if( i == od->tbar_side )
-				set_astbar_focused( asw->tbar, update_canvas, focused );
+		if( i == od->tbar_side )
+			set_astbar_focused( asw->tbar, update_canvas, focused );
 
-			for( k = 0 ; k < FRAME_PARTS ; ++k )
-				if( od->tbar2canvas_xref[k] == i )
-					set_astbar_focused( asw->frame_bars[k], update_canvas, focused );
-		}
-    	/* now posting all the changes on display :*/
-    	for( i = FRAME_SIDES; --i >= 0; )
-        	if( is_canvas_dirty(asw->frame_sides[i]) )
-            	update_canvas_display( asw->frame_sides[i] );
-        if( asw->internal && asw->internal->on_hilite_changed )
-            asw->internal->on_hilite_changed( asw->internal, NULL, focused );
-        if( ASWIN_GET_FLAGS( asw, AS_ShapedDecor ) )
-            SetShape( asw, 0 );
-    }else /* Iconic !!! */
-    {
+		for( k = 0 ; k < FRAME_PARTS ; ++k )
+			if( od->tbar2canvas_xref[k] == i )
+				set_astbar_focused( asw->frame_bars[k], update_canvas, focused );
+	}
+    /* now posting all the changes on display :*/
+    for( i = FRAME_SIDES; --i >= 0; )
+        if( is_canvas_dirty(asw->frame_sides[i]) )
+            update_canvas_display( asw->frame_sides[i] );
+    if( asw->internal && asw->internal->on_hilite_changed )
+        asw->internal->on_hilite_changed( asw->internal, NULL, focused );
+    if( ASWIN_GET_FLAGS( asw, AS_ShapedDecor ) )
+        SetShape( asw, 0 );
+    if(!ASWIN_GET_FLAGS(asw, AS_Iconic))
+	{	
         set_astbar_focused( asw->icon_button, asw->icon_canvas, focused );
         set_astbar_focused( asw->icon_title, asw->icon_title_canvas, focused );
         if( is_canvas_dirty(asw->icon_canvas) )
@@ -1424,6 +1454,10 @@ LOCAL_DEBUG_CALLER_OUT( "client = %p, iconify = %d", asw, iconify );
         else if( asw->icon_title_canvas )
             asw->status->icon_window = asw->icon_title_canvas->w ;
 
+		LOCAL_DEBUG_OUT( "hilited == %p", Scr.Windows->hilited );
+    	if( Scr.Windows->hilited == asw )
+			hide_hilite();
+
         if (get_flags(Scr.Feel.flags, ClickToFocus) || get_flags(Scr.Feel.flags, SloppyFocus))
         {
             if (asw == Scr.Windows->focused)
@@ -1504,7 +1538,7 @@ Bool
 bring_aswindow_on_vscreen( ASWindow *asw )
 {
     int min_x, min_y, max_x, max_y ;
-    int margin = Scr.MyDisplayWidth>>5 ;
+    int margin_x = Scr.MyDisplayWidth>>5, margin_y = Scr.MyDisplayHeight>>5 ;
 
     if( asw == NULL )
         return False;
@@ -1526,15 +1560,31 @@ bring_aswindow_on_vscreen( ASWindow *asw )
     {
         int new_x = asw->status->x ;
         int new_y = asw->status->y ;
-        if( asw->status->x + (int)asw->status->width < min_x + margin )
-            new_x = min_x + margin - (int)asw->status->width ;
-        else if( asw->status->x > max_x - margin )
-            new_x = max_x - margin ;
+		int w = asw->status->width ;
+		int h = asw->status->height ;
 
-        if( asw->status->y + (int)asw->status->height < min_y + margin )
-            new_y = min_y + margin - (int)asw->status->height ;
-        else if( asw->status->y > max_y - margin )
-            new_y = max_y - margin ;
+		if( margin_x > w>>2 ) 
+		{	
+			margin_x = w>>2 ;
+			if( margin_x == 0 ) 
+				margin_x = 1 ;
+		}
+		if( margin_y > h>>2 ) 
+		{	
+			margin_y = h>>2 ;
+			if( margin_y == 0 ) 
+				margin_y = 1 ;
+		}
+
+        if( new_x + w < min_x + margin_x )
+            new_x = min_x + margin_x - w ;
+        else if( new_x > max_x - margin_x )
+            new_x = max_x - margin_x ;
+
+        if( new_y + h < min_y + margin_y )
+            new_y = min_y + margin_y - h ;
+        else if( new_y > max_y - margin_y )
+            new_y = max_y - margin_y ;
         LOCAL_DEBUG_OUT( "min_pos = (%+d%+d), max_pos = (%+d%+d), new_pos = (%+d%+d)", min_x, min_y, max_x, max_y, new_x, new_y );
         if( new_x != asw->status->x || new_y != asw->status->y )
             moveresize_aswindow_wm( asw, new_x, new_y, asw->status->width, asw->status->height, False );
@@ -1807,7 +1857,7 @@ hilite_aswindow( ASWindow *asw )
             on_window_hilite_changed (asw, True);
         
 		if( Scr.Windows->hilited )
-			broadcast_focus_change( asw, False );
+			broadcast_focus_change( Scr.Windows->hilited, False );
 		if( asw )
 			broadcast_focus_change( asw, True );
 
